@@ -43,7 +43,6 @@ use debugid::{CodeId, DebugId};
 use futures_util::lock::Mutex as FutMutex;
 use tracing::trace;
 
-use std::boxed::Box;
 use std::collections::HashMap;
 use std::fs;
 use std::path::PathBuf;
@@ -172,9 +171,7 @@ impl Module for SimpleModule {
 
 /// Like `PathBuf::file_name`, but try to work on Windows or POSIX-style paths.
 fn leafname(path: &str) -> &str {
-    path.rsplit(|c| c == '/' || c == '\\')
-        .next()
-        .unwrap_or(path)
+    path.rsplit(['/', '\\']).next().unwrap_or(path)
 }
 
 /// If `filename` ends with `match_extension`, remove it. Append `new_extension` to the result.
@@ -194,6 +191,8 @@ fn replace_or_add_extension(filename: &str, match_extension: &str, new_extension
 /// A lookup we would like to perform for some file (sym, exe, pdb, dll, ...)
 #[derive(Debug, Clone)]
 pub struct FileLookup {
+    pub debug_id: String,
+    pub debug_file: String,
     pub cache_rel: String,
     pub server_rel: String,
 }
@@ -225,6 +224,8 @@ pub fn breakpad_sym_lookup(module: &(dyn Module + Sync)) -> Option<FileLookup> {
     Some(FileLookup {
         cache_rel: rel_path.clone(),
         server_rel: rel_path,
+        debug_id: debug_id.breakpad().to_string(),
+        debug_file: filename,
     })
 }
 
@@ -268,6 +269,8 @@ pub fn extra_debuginfo_lookup(module: &(dyn Module + Sync)) -> Option<FileLookup
     Some(FileLookup {
         cache_rel: rel_path.clone(),
         server_rel: rel_path,
+        debug_id: debug_id.to_string(),
+        debug_file: leaf.to_string(),
     })
 }
 
@@ -288,6 +291,8 @@ pub fn binary_lookup(module: &(dyn Module + Sync)) -> Option<FileLookup> {
     Some(FileLookup {
         cache_rel: [debug_leaf, &debug_id.breakpad().to_string(), bin_leaf].join("/"),
         server_rel: [bin_leaf, code_id.as_ref(), bin_leaf].join("/"),
+        debug_id: debug_id.to_string(),
+        debug_file: debug_file.to_string(),
     })
 }
 
@@ -471,10 +476,19 @@ impl SymbolSupplier for SimpleSymbolSupplier {
         trace!("SimpleSymbolSupplier search");
         if let Some(lookup) = lookup(module, file_kind) {
             for path in self.paths.iter() {
-                let test_path = path.join(lookup.cache_rel.clone());
-                if fs::metadata(&test_path).ok().map_or(false, |m| m.is_file()) {
-                    trace!("SimpleSymbolSupplier found file {}", test_path.display());
-                    return Ok(test_path);
+                if path.is_file() && file_kind == FileKind::BreakpadSym {
+                    if let Ok(sf) = SymbolFile::from_file(path) {
+                        if sf.module_id == lookup.debug_id {
+                            trace!("SimpleSymbolSupplier found file {}", path.display());
+                            return Ok(path.to_path_buf());
+                        }
+                    }
+                } else if path.is_dir() {
+                    let test_path = path.join(lookup.cache_rel.clone());
+                    if fs::metadata(&test_path).ok().map_or(false, |m| m.is_file()) {
+                        trace!("SimpleSymbolSupplier found file {}", test_path.display());
+                        return Ok(test_path);
+                    }
                 }
             }
         } else {
@@ -845,10 +859,10 @@ impl Symbolizer {
                 let mut stats = SymbolStats::default();
                 match &result {
                     Ok(res) => {
-                        stats.symbol_url = res.symbols.url.clone();
+                        stats.symbol_url.clone_from(&res.symbols.url);
                         stats.loaded_symbols = true;
                         stats.corrupt_symbols = false;
-                        stats.extra_debug_info = res.extra_debug_info.clone();
+                        stats.extra_debug_info.clone_from(&res.extra_debug_info);
                     }
                     Err(SymbolError::NotFound) => {
                         stats.loaded_symbols = false;
@@ -916,10 +930,9 @@ fn test_replace_or_add_extension() {
 mod test {
 
     use super::*;
-    use std::fs;
     use std::fs::File;
     use std::io::Write;
-    use std::path::{Path, PathBuf};
+    use std::path::Path;
     use std::str::FromStr;
 
     #[tokio::test]
