@@ -8,14 +8,16 @@ use std::sync::{Arc, Mutex};
 use std::time::{Duration, SystemTime};
 
 use minidump::system_info::PointerWidth;
-use minidump::{self, *};
+use minidump::*;
 use minidump_unwind::{
     walk_stack, CallStack, CallStackInfo, FrameTrust, StackFrame, SymbolProvider, SystemInfo,
 };
 
-use crate::op_analysis::MemoryAccess;
+use crate::op_analysis::MemoryAddressInfo;
 use crate::process_state::{LinuxStandardBase, ProcessState};
-use crate::{arg_recovery, evil, AdjustedAddress, LinuxProcLimits, LinuxProcStatus};
+use crate::{
+    arg_recovery, evil, AdjustedAddress, CrashInconsistency, LinuxProcLimits, LinuxProcStatus,
+};
 
 /// Configuration of the processor's exact behaviour.
 ///
@@ -453,6 +455,7 @@ where
     if let Some(details) = &mut exception_details {
         info.check_for_bitflips(details);
         info.check_for_guard_pages(details);
+        info.check_for_crash_inconsistencies(details);
     }
     info.into_process_state(dump, symbol_provider, exception_details)
         .await
@@ -480,6 +483,7 @@ struct MinidumpInfo<'a> {
     memory_info_list: Option<MinidumpMemoryInfoList<'a>>,
     linux_maps: Option<MinidumpLinuxMaps<'a>>,
     */
+    linux_memory_map_count: Option<usize>,
     memory_info: UnifiedMemoryInfoList<'a>,
     handle_data_stream: Option<MinidumpHandleDataStream>,
     exception: Option<MinidumpException<'a>>,
@@ -584,6 +588,7 @@ impl<'a> MinidumpInfo<'a> {
         let memory_list = dump.get_memory().unwrap_or_default();
         let memory_info_list = dump.get_stream::<MinidumpMemoryInfoList>().ok();
         let linux_maps = dump.get_stream::<MinidumpLinuxMaps>().ok();
+        let linux_memory_map_count = linux_maps.clone().map(|maps| maps.memory_map_count());
         let memory_info =
             UnifiedMemoryInfoList::new(memory_info_list, linux_maps).unwrap_or_default();
         let handle_data_stream = dump.get_stream::<MinidumpHandleDataStream>().ok();
@@ -614,6 +619,7 @@ impl<'a> MinidumpInfo<'a> {
             linux_maps: Option<MinidumpLinuxMaps<'a>>,
             */
             memory_info,
+            linux_memory_map_count,
             handle_data_stream,
             exception,
             //exception_details: None,
@@ -622,6 +628,8 @@ impl<'a> MinidumpInfo<'a> {
 
     /// Get details about the minidump exception, if available.
     pub fn get_exception_details(&self) -> Option<ExceptionDetails<'a>> {
+        use crate::op_analysis::InstructionPointerUpdate;
+
         let exception = self.exception.as_ref()?;
 
         let reason = exception.get_crash_reason(self.system_info.os, self.system_info.cpu);
@@ -645,29 +653,43 @@ impl<'a> MinidumpInfo<'a> {
                 stack_memory_ref,
             ) {
                 Ok(op_analysis) => {
-                    let memory_accesses = op_analysis.memory_accesses.as_deref();
+                    let access_addresses =
+                        op_analysis.memory_access_list.as_ref().map(|access_list| {
+                            access_list
+                                .iter()
+                                .map(|access| access.address_info)
+                                .collect::<Vec<MemoryAddressInfo>>()
+                        });
+                    let addresses = access_addresses.map(|mut accesses| {
+                        match op_analysis.instruction_pointer_update {
+                            Some(InstructionPointerUpdate::Update { address_info }) => {
+                                accesses.push(address_info);
+                                accesses
+                            }
+                            _ => accesses,
+                        }
+                    });
+                    let addresses = addresses.as_deref();
 
-                    let adjusted_address = try_detect_null_pointer_in_disguise(memory_accesses)
+                    let adjusted_address = try_detect_null_pointer_in_disguise(addresses)
                         .map(|offset| AdjustedAddress::NullPointerWithOffset(offset.into()))
                         .or_else(|| {
                             try_get_non_canonical_crash_address(
                                 &self.system_info,
-                                memory_accesses,
+                                addresses,
                                 reason,
                                 address,
                             )
                             .map(|addr| AdjustedAddress::NonCanonical(addr.into()))
                         });
 
-                    exception_info = Some(crate::ExceptionInfo {
+                    instruction_registers.clone_from(&op_analysis.registers);
+                    exception_info = Some(crate::ExceptionInfo::with_op_analysis(
                         reason,
-                        address: address.into(),
+                        address.into(),
                         adjusted_address,
-                        instruction_str: Some(op_analysis.instruction_str),
-                        memory_accesses: op_analysis.memory_accesses,
-                        possible_bit_flips: Default::default(),
-                    });
-                    instruction_registers = op_analysis.registers;
+                        op_analysis,
+                    ));
                 }
                 Err(e) => {
                     tracing::warn!("failed to analyze the thread context: {e}");
@@ -675,14 +697,8 @@ impl<'a> MinidumpInfo<'a> {
             }
         }
 
-        let info = exception_info.unwrap_or_else(|| crate::ExceptionInfo {
-            reason,
-            address: address.into(),
-            adjusted_address: None,
-            instruction_str: None,
-            memory_accesses: None,
-            possible_bit_flips: Default::default(),
-        });
+        let info =
+            exception_info.unwrap_or_else(|| crate::ExceptionInfo::new(reason, address.into()));
 
         Some(ExceptionDetails {
             info,
@@ -712,6 +728,7 @@ impl<'a> MinidumpInfo<'a> {
         let info = &mut exception_details.info;
 
         use bitflip::BitRange;
+        use memory_operation::MemoryOperation;
         let bit_flip_address = match &info.adjusted_address {
             // Use the non canonical address if present.
             Some(AdjustedAddress::NonCanonical(v)) => Some((v.0, BitRange::Amd64NonCanonical)),
@@ -728,7 +745,7 @@ impl<'a> MinidumpInfo<'a> {
             )),
         };
         if let Some((address, bit_range)) = bit_flip_address {
-            let memory_op = bitflip::MemoryOperation::from_crash_reason(&info.reason);
+            let memory_op = MemoryOperation::from_crash_reason(&info.reason);
             info.possible_bit_flips = bitflip::try_bit_flips(
                 address,
                 None,
@@ -765,9 +782,12 @@ impl<'a> MinidumpInfo<'a> {
     pub fn check_for_guard_pages(&self, exception_details: &mut ExceptionDetails<'a>) {
         const GUARD_MEMORY_MAX_SIZE: u64 = 2 << 14;
 
-        if let Some(accesses) = &mut exception_details.info.memory_accesses {
-            for access in accesses {
-                let Some(info) = self.memory_info.memory_info_at_address(access.address) else {
+        if let Some(access_list) = &mut exception_details.info.memory_access_list {
+            for access in &mut access_list.accesses {
+                let Some(info) = self
+                    .memory_info
+                    .memory_info_at_address(access.address_info.address)
+                else {
                     continue;
                 };
                 let Some(range) = info.memory_range() else {
@@ -803,12 +823,192 @@ impl<'a> MinidumpInfo<'a> {
                     && range.end - range.start < GUARD_MEMORY_MAX_SIZE
                     && is_adjacent_to_accessible_memory()
                 {
-                    access.is_likely_guard_page = true;
+                    access.address_info.is_likely_guard_page = true;
                 }
             }
         }
     }
 
+    /// Check for inconsistencies between crash reason and crashing instruction
+    pub fn check_for_crash_inconsistencies(&self, exception_details: &mut ExceptionDetails) {
+        use minidump_common::errors::{
+            ExceptionCodeLinuxSigfpeKind as LinuxSigfpe,
+            ExceptionCodeMacArithmeticPpcType as MacArithPpc,
+            ExceptionCodeMacArithmeticX86Type as MacArithX86, ExceptionCodeWindows,
+            ExceptionCodeWindowsAccessType as WinAccess, NtStatusWindows,
+        };
+        use CrashInconsistency as Inconsistency;
+
+        let mut inconsistencies = Vec::new();
+        match exception_details.info.reason {
+            CrashReason::MacArithmeticPpc(MacArithPpc::EXC_PPC_ZERO_DIVIDE)
+            | CrashReason::MacArithmeticX86(MacArithX86::EXC_I386_DIV)
+            | CrashReason::LinuxSigfpe(LinuxSigfpe::FPE_INTDIV)
+            | CrashReason::WindowsGeneral(ExceptionCodeWindows::EXCEPTION_INT_DIVIDE_BY_ZERO) => {
+                if exception_details
+                    .info
+                    .instruction_properties
+                    .as_ref()
+                    .is_some_and(|p| !p.is_division)
+                {
+                    inconsistencies.push(Inconsistency::IntDivByZeroNotPossible);
+                }
+            }
+
+            CrashReason::WindowsGeneral(ExceptionCodeWindows::EXCEPTION_PRIV_INSTRUCTION)
+            | CrashReason::WindowsNtStatus(NtStatusWindows::STATUS_PRIVILEGED_INSTRUCTION) => {
+                if exception_details
+                    .info
+                    .instruction_properties
+                    .as_ref()
+                    .is_some_and(|p| !p.is_privileged)
+                {
+                    inconsistencies.push(Inconsistency::PrivInstructionCrashWithoutPrivInstruction);
+                }
+            }
+
+            CrashReason::WindowsAccessViolation(WinAccess::READ) => {
+                // To conclude with certainty that it is a non-canonical access exception,
+                // need to check if crashing access (READ 0xffffffffffffffff) is actually not among accesses
+                let is_gpf = represents_general_protection_fault(
+                    self.system_info.os,
+                    exception_details.info.reason,
+                    exception_details.info.address.0,
+                ) && self.crashing_access_is_not_among_accesses(exception_details);
+                let gpf_implies_non_canonical = exception_details
+                    .info
+                    .instruction_properties
+                    .as_ref()
+                    .is_some_and(|p| p.is_only_gpf_when_non_canonical);
+
+                if is_gpf {
+                    if gpf_implies_non_canonical
+                        && self.non_canonical_address_is_not_among_accesses(exception_details)
+                    {
+                        inconsistencies.push(Inconsistency::NonCanonicalAddressFalselyReported);
+                    }
+                } else {
+                    if self.crashing_access_is_not_among_accesses(exception_details) {
+                        inconsistencies.push(Inconsistency::CrashingAccessNotFoundInMemoryAccesses);
+                    }
+                    if self.access_is_not_violation(exception_details) {
+                        inconsistencies.push(Inconsistency::AccessViolationWhenAccessAllowed);
+                    }
+                }
+            }
+            // We treat stack overflow like an access violation with unknown operation
+            // i.e. It is considered inconsistent if the instruction has no memory access
+            CrashReason::WindowsGeneral(ExceptionCodeWindows::EXCEPTION_STACK_OVERFLOW)
+            | CrashReason::WindowsAccessViolation(WinAccess::WRITE)
+            | CrashReason::WindowsAccessViolation(WinAccess::EXEC) => {
+                if self.crashing_access_is_not_among_accesses(exception_details) {
+                    inconsistencies.push(Inconsistency::CrashingAccessNotFoundInMemoryAccesses);
+                }
+                if self.access_is_not_violation(exception_details) {
+                    inconsistencies.push(Inconsistency::AccessViolationWhenAccessAllowed);
+                }
+            }
+
+            _ => (),
+        }
+        exception_details.info.inconsistencies = inconsistencies;
+    }
+
+    /// Returns whether non-canonical address is not used in accesses of crashing instruction
+    /// Returns false if there is insufficient information to determine
+    fn non_canonical_address_is_not_among_accesses(
+        &self,
+        exception_details: &ExceptionDetails,
+    ) -> bool {
+        use crate::op_analysis::InstructionPointerUpdate;
+        const NON_CANONICAL_RANGE: RangeInclusive<u64> =
+            0x0000_8000_0000_0000..=0xffff_7fff_ffff_ffff;
+        let info = &exception_details.info;
+        // `adjusted_address` might not show that there is non-canonical address if there is
+        // also a disguised null pointer, so we check `memory_access_list` directly
+        info.instruction_properties
+            .as_ref()
+            .is_some_and(|properties| properties.is_access_derivable)
+            && info.memory_access_list.as_ref().is_some_and(|access_list| {
+                !access_list
+                    .iter()
+                    .any(|access| NON_CANONICAL_RANGE.contains(&access.address_info.address))
+            })
+            && info
+                .instruction_pointer_update
+                .as_ref()
+                .is_some_and(|update| match update {
+                    InstructionPointerUpdate::Update { address_info } => {
+                        !NON_CANONICAL_RANGE.contains(&address_info.address)
+                    }
+                    InstructionPointerUpdate::NoUpdate => true,
+                })
+    }
+
+    /// Returns whether crashing access is not found in accesses of crashing instruction
+    /// Returns false if there is insufficient information to determine
+    fn crashing_access_is_not_among_accesses(&self, exception_details: &ExceptionDetails) -> bool {
+        use crate::op_analysis::MemoryAccessType;
+        use minidump_common::errors::{
+            ExceptionCodeWindows, ExceptionCodeWindowsAccessType as WinAccess,
+        };
+
+        let info = &exception_details.info;
+        let crash_address = info.address.0;
+        let has_access_derivable_instruction = info
+            .instruction_properties
+            .as_ref()
+            .is_some_and(|p| p.is_access_derivable);
+
+        if let Some(access_list) = &info.memory_access_list {
+            match info.reason {
+                CrashReason::WindowsAccessViolation(WinAccess::READ) => {
+                    has_access_derivable_instruction
+                        && !access_list.contains_access(crash_address, MemoryAccessType::Read)
+                        && !access_list.contains_access(crash_address, MemoryAccessType::ReadWrite)
+                }
+                CrashReason::WindowsAccessViolation(WinAccess::WRITE) => {
+                    has_access_derivable_instruction
+                        && !access_list.contains_access(crash_address, MemoryAccessType::Write)
+                        && !access_list.contains_access(crash_address, MemoryAccessType::ReadWrite)
+                }
+                CrashReason::WindowsAccessViolation(WinAccess::EXEC) => exception_details
+                    .context
+                    .as_ref()
+                    .is_some_and(|context| context.get_instruction_pointer() != crash_address),
+                CrashReason::WindowsGeneral(ExceptionCodeWindows::EXCEPTION_STACK_OVERFLOW) => {
+                    has_access_derivable_instruction && access_list.is_empty()
+                }
+                _ => false,
+            }
+        } else {
+            false
+        }
+    }
+
+    /// Returns whether crashing access is allowed by the memory info
+    /// Returns false if there is insufficient information to determine
+    fn access_is_not_violation(&self, exception_details: &ExceptionDetails) -> bool {
+        {
+            use memory_operation::MemoryOperation;
+            use minidump_common::errors::ExceptionCodeWindowsAccessType as WinAccess;
+
+            let info = &exception_details.info;
+            let crash_address = info.address.0;
+
+            if let Some(mi) = self.memory_info.memory_info_at_address(crash_address) {
+                matches!(
+                    info.reason,
+                    CrashReason::WindowsAccessViolation(WinAccess::READ)
+                        | CrashReason::WindowsAccessViolation(WinAccess::WRITE)
+                        | CrashReason::WindowsAccessViolation(WinAccess::EXEC)
+                ) && MemoryOperation::from_crash_reason(&info.reason).is_allowed_for(&mi)
+            } else {
+                false
+            }
+        }
+    }
+
     pub async fn into_process_state<P, T>(
         self,
         dump: &Minidump<'a, T>,
@@ -919,6 +1119,7 @@ impl<'a> MinidumpInfo<'a> {
             unknown_streams,
             unimplemented_streams,
             symbol_stats,
+            linux_memory_map_count: self.linux_memory_map_count,
         };
 
         // Report the unwalked result
@@ -1016,6 +1217,41 @@ impl<'a> MinidumpInfo<'a> {
     }
 }
 
+impl crate::ExceptionInfo {
+    fn new(reason: CrashReason, address: crate::Address) -> Self {
+        Self {
+            reason,
+            address,
+            adjusted_address: None,
+            instruction_str: None,
+            instruction_properties: None,
+            memory_access_list: None,
+            instruction_pointer_update: None,
+            possible_bit_flips: Default::default(),
+            inconsistencies: Default::default(),
+        }
+    }
+
+    fn with_op_analysis(
+        reason: CrashReason,
+        address: crate::Address,
+        adjusted_address: Option<AdjustedAddress>,
+        op_analysis: crate::op_analysis::OpAnalysis,
+    ) -> Self {
+        Self {
+            reason,
+            address,
+            adjusted_address,
+            instruction_str: Some(op_analysis.instruction_str),
+            instruction_properties: Some(op_analysis.instruction_properties),
+            memory_access_list: op_analysis.memory_access_list,
+            instruction_pointer_update: op_analysis.instruction_pointer_update,
+            possible_bit_flips: Default::default(),
+            inconsistencies: Default::default(),
+        }
+    }
+}
+
 struct ExceptionDetails<'a> {
     info: crate::ExceptionInfo,
     context: Option<std::borrow::Cow<'a, MinidumpContext>>,
@@ -1039,7 +1275,7 @@ struct ExceptionDetails<'a> {
 /// determined, `None` otherwise
 fn try_get_non_canonical_crash_address(
     system_info: &SystemInfo,
-    memory_accesses: Option<&[MemoryAccess]>,
+    memory_addresses: Option<&[MemoryAddressInfo]>,
     reason: CrashReason,
     address: u64,
 ) -> Option<u64> {
@@ -1054,12 +1290,12 @@ fn try_get_non_canonical_crash_address(
         return None;
     }
 
-    if !is_non_canonical_exception(system_info.os, reason, address) {
+    if !represents_general_protection_fault(system_info.os, reason, address) {
         return None;
     }
 
     // If we weren't able to determine the memory accessed by the instruction, we can't do this analysis
-    if memory_accesses.is_none() {
+    if memory_addresses.is_none() {
         tracing::warn!(
             "lack of instruction analysis prevented determination of non-canonical address"
         );
@@ -1067,23 +1303,35 @@ fn try_get_non_canonical_crash_address(
     }
 
     // If any of the instructions operands were within the non-canonical range, we have our culprit
-    for access in memory_accesses.unwrap().iter() {
+    for access in memory_addresses.unwrap().iter() {
         if NON_CANONICAL_RANGE.contains(&access.address) {
             return Some(access.address);
         }
     }
 
-    tracing::warn!("somehow got a non-canonical address exception in an instruction that doesn't appear to access one");
+    tracing::warn!(
+        r#"somehow got a general protection fault in an instruction 
+        that doesn't appear to access a non-canonical address"#
+    );
 
     None
 }
 
-/// Report whether the given exception represents a non-canonical access on the given OS
+/// Report whether the given exception represents a general protection fault (GPF) on the given OS
 ///
-/// Different operating systems have different ways of reporting non-canonical address accesses
-/// This function will return whether the given `exception_info` object represents such an access
-/// on the given OS
-fn is_non_canonical_exception(os: system_info::Os, reason: CrashReason, address: u64) -> bool {
+/// Different operating systems have different ways of reporting GPF, this function will return
+/// whether the given `exception_info` object represents that on the given OS
+///
+/// Note it is possible that the crashing instruction is performing an access to 0xffffffffffffffff
+/// or 0x00000000000000000, and the crash is actually not caused by GPF
+///
+/// The most likely cause of GPF is the use of non-canonical address but there are other possibilities,
+/// such as misaligned accesses
+fn represents_general_protection_fault(
+    os: system_info::Os,
+    reason: CrashReason,
+    address: u64,
+) -> bool {
     use minidump_common::errors as minidump_errors;
     use system_info::Os;
 
@@ -1133,9 +1381,11 @@ fn is_non_canonical_exception(os: system_info::Os, reason: CrashReason, address:
 /// This function will search though all the memory accessses for the instruction and see if any
 /// of them were flagged by `op_analysis` as being a disguised nullptr. If so, we return that
 /// address as an "offset" from the null pointer value
-fn try_detect_null_pointer_in_disguise(memory_accesses: Option<&[MemoryAccess]>) -> Option<u64> {
-    if let Some(memory_accesses) = memory_accesses {
-        for access in memory_accesses.iter() {
+fn try_detect_null_pointer_in_disguise(
+    memory_addresses: Option<&[MemoryAddressInfo]>,
+) -> Option<u64> {
+    if let Some(memory_addresses) = memory_addresses {
+        for access in memory_addresses.iter() {
             if access.is_likely_null_pointer_dereference {
                 return Some(access.address);
             }
@@ -1144,44 +1394,71 @@ fn try_detect_null_pointer_in_disguise(memory_accesses: Option<&[MemoryAccess]>)
     None
 }
 
-/// Bit-flip detection.
-mod bitflip {
+pub mod memory_operation {
     use super::*;
-    use crate::PossibleBitFlip;
 
     /// The memory operation occurring when a crash occurred.
-    #[derive(Default, PartialEq, Eq, Clone, Copy)]
+    #[derive(Clone, Copy, Debug, Default, Eq, PartialEq)]
     pub enum MemoryOperation {
         #[default]
-        Unknown,
+        Undetermined,
         Read,
         Write,
         Execute,
     }
 
+    impl std::fmt::Display for MemoryOperation {
+        fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+            f.write_str(match self {
+                Self::Undetermined => "Undetermined",
+                Self::Read => "Read",
+                Self::Write => "Write",
+                Self::Execute => "Execute",
+            })
+        }
+    }
+
     impl MemoryOperation {
         pub fn from_crash_reason(reason: &CrashReason) -> Self {
-            // TODO: it may be possible to derive the read/write/exec when disassembling the faulting
-            // instruction, though this may be fairly verbose to implement.
             use minidump_common::errors::ExceptionCodeWindowsAccessType as WinAccess;
+
             match reason {
-                CrashReason::WindowsAccessViolation(WinAccess::READ) => MemoryOperation::Read,
-                CrashReason::WindowsAccessViolation(WinAccess::WRITE) => MemoryOperation::Write,
-                CrashReason::WindowsAccessViolation(WinAccess::EXEC) => MemoryOperation::Execute,
+                CrashReason::WindowsAccessViolation(WinAccess::READ) => Self::Read,
+                CrashReason::WindowsAccessViolation(WinAccess::WRITE) => Self::Write,
+                CrashReason::WindowsAccessViolation(WinAccess::EXEC) => Self::Execute,
                 _ => Self::default(),
             }
         }
 
-        /// Return whether this memory operation is allowed in the given memory region.
-        pub fn allowed_for(&self, memory_info: &UnifiedMemoryInfo) -> bool {
+        /// Return whether this memory operation is possibily allowed in the given memory region.
+        /// If operation is `Undetermined`, this method returns true.
+        pub fn is_possibly_allowed_for(&self, memory_info: &UnifiedMemoryInfo) -> bool {
             match self {
-                Self::Unknown => true,
+                Self::Undetermined => true,
+                Self::Read => memory_info.is_readable(),
+                Self::Write => memory_info.is_writable(),
+                Self::Execute => memory_info.is_executable(),
+            }
+        }
+
+        /// Return whether this memory operation is definitely allowed in the given memory region.
+        /// If operation is `Undetermined`, this method returns false.
+        pub fn is_allowed_for(&self, memory_info: &UnifiedMemoryInfo) -> bool {
+            match self {
+                Self::Undetermined => false,
                 Self::Read => memory_info.is_readable(),
                 Self::Write => memory_info.is_writable(),
                 Self::Execute => memory_info.is_executable(),
             }
         }
     }
+}
+
+/// Bit-flip detection.
+mod bitflip {
+    use super::*;
+    use crate::memory_operation::MemoryOperation;
+    use crate::PossibleBitFlip;
 
     /// The bit range over which to check bit flips.
     #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -1204,7 +1481,7 @@ mod bitflip {
     /// Try to determine whether an address was the result of a flipped bit.
     ///
     /// `memory_operation` represents the memory operation that was occurring at the crashing address
-    /// (read/write/exec). If left as `Unknown`, all memory operations are considered allowed.
+    /// (read/write/exec). If left as `Undetermined`, all memory operations are considered allowed.
     /// Otherwise, specify one of the operations that was occurring.
     pub fn try_bit_flips(
         address: u64,
@@ -1217,7 +1494,7 @@ mod bitflip {
         let mut addresses = Vec::new();
         // If the address maps to valid memory, don't do anything else.
         if let Some(mi) = memory_info.memory_info_at_address(address) {
-            if memory_operation.allowed_for(&mi) {
+            if memory_operation.is_possibly_allowed_for(&mi) {
                 return addresses;
             }
         }
@@ -1240,8 +1517,8 @@ mod bitflip {
                 addresses.push(create_possible_address(possible_address));
             }
             if let Some(mi) = memory_info.memory_info_at_address(possible_address) {
-                if memory_operation.allowed_for(&mi) {
-                    addresses.push(create_possible_address(possible_address));
+                if memory_operation.is_possibly_allowed_for(&mi) {
+                    addresses.push(create_possible_address(possible_address))
                 }
             }
         }
