@@ -846,3 +846,42 @@ fn test_macos_inlines_human() {
     insta::assert_snapshot!(stdout);
     assert_eq!(stderr, "");
 }
+
+#[test]
+fn test_linux_json_pretty() {
+    let bin = env!("CARGO_BIN_EXE_minidump-stackwalk");
+    let output = Command::new(bin)
+        .arg("--json")
+        .arg("--pretty")
+        .arg("../testdata/linux-mini.dmp")
+        .stdout(Stdio::piped())
+        .stderr(Stdio::piped())
+        .output()
+        .unwrap();
+
+    let stdout = String::from_utf8(output.stdout).unwrap();
+    let stderr = String::from_utf8(output.stderr).unwrap();
+
+    assert!(output.status.success());
+    insta::assert_snapshot!(stdout);
+    assert_eq!(stderr, "");
+}
+
+#[test]
+fn test_linux_human() {
+    let bin = env!("CARGO_BIN_EXE_minidump-stackwalk");
+    let output = Command::new(bin)
+        .arg("--human")
+        .arg("../testdata/linux-mini.dmp")
+        .stdout(Stdio::piped())
+        .stderr(Stdio::piped())
+        .output()
+        .unwrap();
+
+    let stdout = String::from_utf8(output.stdout).unwrap();
+    let stderr = String::from_utf8(output.stderr).unwrap();
+
+    assert!(output.status.success());
+    insta::assert_snapshot!(stdout);
+    assert_eq!(stderr, "");
+}
